@@ -48,6 +48,10 @@ public class LibVLC {
     /** libvlc_media_list_t pointer */
     private long mMediaListInstance = 0; // Read-only, reserved for JNI
 
+    /** Buffer for VLC messages */
+    private StringBuffer mDebugLogBuffer;
+    private boolean mIsBufferingLog = false;
+
     private Aout mAout;
 
     /** Keep screen bright */
@@ -191,6 +195,7 @@ public class LibVLC {
      */
     private void init() throws LibVlcException {
         Log.v(TAG, "Initializing LibVLC");
+        mDebugLogBuffer = new StringBuffer();
         if (!mIsInitialized) {
             if(!Util.hasCompatibleCPU()) {
                 Log.e(TAG, Util.getErrorMsg());
@@ -330,6 +335,23 @@ public class LibVLC {
      */
     private native void nativeDestroy();
 
+    /**
+     * Start buffering to the mDebugLogBuffer.
+     */
+    public native void startDebugBuffer();
+    public native void stopDebugBuffer();
+    public String getBufferContent() {
+        return mDebugLogBuffer.toString();
+    }
+
+    public void clearBuffer() {
+        mDebugLogBuffer.setLength(0);
+    }
+
+    public boolean isDebugBuffering() {
+        return mIsBufferingLog;
+    }
+
     /**
      * Read a media
      * @param instance: the instance of libVLC
