@@ -33,6 +33,8 @@ public class VLCApplication extends Application {
     public final static String TAG = "VLC/VLCApplication";
     private static VLCApplication instance;
 
+    public final static String SLEEP_INTENT = "org.videolan.vlc.SleepIntent";
+
     @Override
     public void onCreate() {
         super.onCreate();
@@ -43,7 +45,9 @@ public class VLCApplication extends Application {
         if (p != null && !p.equals("")) {
             Locale locale;
             // workaround due to region code
-            if(p.startsWith("zh")) {
+            if(p.equals("zh-TW")) {
+                locale = Locale.TRADITIONAL_CHINESE;
+            } else if(p.startsWith("zh")) {
                 locale = Locale.CHINA;
             } else {
                 locale = new Locale(p);
