@@ -34,6 +34,7 @@ import android.graphics.Bitmap;
 import android.graphics.BitmapFactory;
 import android.view.LayoutInflater;
 import android.view.View;
+import android.view.View.OnClickListener;
 import android.view.ViewGroup;
 import android.widget.ArrayAdapter;
 import android.widget.GridView;
@@ -49,9 +50,11 @@ public class VideoListAdapter extends ArrayAdapter<Media>
     private int mSortDirection = 1;
     private int mSortBy = SORT_BY_TITLE;
     private boolean mListMode = false;
+    private VideoGridFragment mFragment;
 
-    public VideoListAdapter(Context context) {
+    public VideoListAdapter(Context context, VideoGridFragment fragment) {
         super(context, 0);
+        mFragment = fragment;
     }
 
     public final static String TAG = "VLC/MediaLibraryAdapter";
@@ -123,15 +126,18 @@ public class VideoListAdapter extends ArrayAdapter<Media>
      * Display the view of a file browser item.
      */
     @Override
-    public View getView(int position, View convertView, ViewGroup parent) {
+    public View getView(final int position, View convertView, ViewGroup parent) {
         ViewHolder holder;
         View v = convertView;
+
         if (v == null || (((ViewHolder)v.getTag()).listmode != mListMode)) {
             LayoutInflater inflater = (LayoutInflater) getContext().getSystemService(Context.LAYOUT_INFLATER_SERVICE);
+
             if (!mListMode)
                 v = inflater.inflate(R.layout.video_grid_item, parent, false);
             else
                 v = inflater.inflate(R.layout.video_list_item, parent, false);
+
             holder = new ViewHolder();
             holder.layout = v.findViewById(R.id.layout_item);
             holder.thumbnail = (ImageView) v.findViewById(R.id.ml_item_thumbnail);
@@ -140,14 +146,16 @@ public class VideoListAdapter extends ArrayAdapter<Media>
             holder.progress = (ProgressBar) v.findViewById(R.id.ml_item_progress);
             holder.listmode = mListMode;
             v.setTag(holder);
+
             /* Set the layoutParams based on the values set in the video_grid_item.xml root element */
             v.setLayoutParams(new GridView.LayoutParams(v.getLayoutParams().width, v.getLayoutParams().height));
-        } else
+        } else {
             holder = (ViewHolder) v.getTag();
+        }
 
         Media media = getItem(position);
-        holder.title.setText(media.getTitle());
 
+        /* Thumbnail */
         if (media.getPicture() != null) {
             //FIXME Warning: the thumbnails are upscaled in the grid view!
             final Bitmap thumbnail = media.getPicture();
@@ -163,9 +171,11 @@ public class VideoListAdapter extends ArrayAdapter<Media>
             holder.thumbnail.setImageBitmap(bitmap);
         }
 
+        /* Color state */
         ColorStateList titleColor = v.getResources().getColorStateList(R.color.list_title);
         holder.title.setTextColor(titleColor);
 
+        /* Time / Duration */
         long lastTime = media.getTime();
         String text;
         if (lastTime > 0) {
@@ -185,7 +195,20 @@ public class VideoListAdapter extends ArrayAdapter<Media>
             text += String.format(" - %dx%d", media.getWidth(), media.getHeight());
         }
 
+        /* Text */
         holder.subtitle.setText(text);
+        holder.title.setText(media.getTitle());
+
+        /* Popup menu */
+        final ImageView more = (ImageView)v.findViewById(R.id.ml_item_more);
+        if (more != null && mFragment != null) {
+            more.setOnClickListener(new OnClickListener() {
+                @Override
+                public void onClick(View v) {
+                    mFragment.onContextPopupMenu(more, position);
+                }
+            });
+        }
 
         return v;
     }
