@@ -113,7 +113,8 @@ public class VideoPlayerActivity extends Activity {
     private View mOverlayHeader;
     private View mOverlayLock;
     private View mOverlayOption;
-    private View mOverlay;
+    private View mOverlayProgress;
+    private View mOverlayInterface;
     private static final int OVERLAY_TIMEOUT = 4000;
     private static final int OVERLAY_INFINITE = 3600000;
     private static final int FADE_OUT = 1;
@@ -204,7 +205,8 @@ public class VideoPlayerActivity extends Activity {
         mOverlayHeader = findViewById(R.id.player_overlay_header);
         mOverlayLock = findViewById(R.id.lock_overlay);
         mOverlayOption = findViewById(R.id.option_overlay);
-        mOverlay = findViewById(R.id.player_overlay);
+        mOverlayProgress = findViewById(R.id.progress_overlay);
+        mOverlayInterface = findViewById(R.id.interface_overlay);
 
         /* header */
         mTitle = (TextView) findViewById(R.id.player_overlay_title);
@@ -221,7 +223,7 @@ public class VideoPlayerActivity extends Activity {
         mInfo = (TextView) findViewById(R.id.player_overlay_info);
 
         mEnableWheelbar = pref.getBoolean("enable_wheel_bar", false);
-        mEnableBrightnessGesture = pref.getBoolean("enable_gesture_brightness", true);
+        mEnableBrightnessGesture = pref.getBoolean("enable_brightness_gesture", true);
         mScreenOrientation = Integer.valueOf(
                 pref.getString("screen_orientation_value", "4" /*SCREEN_ORIENTATION_SENSOR*/));
 
@@ -512,6 +514,9 @@ public class VideoPlayerActivity extends Activity {
             setRequestedOrientation(getScreenOrientation());
         showInfo(R.string.locked, 1000);
         mLock.setBackgroundResource(R.drawable.ic_lock_glow);
+        mTime.setEnabled(false);
+        mSeekbar.setEnabled(false);
+        mLength.setEnabled(false);
         hideOverlay(true);
     }
 
@@ -523,6 +528,9 @@ public class VideoPlayerActivity extends Activity {
             setRequestedOrientation(ActivityInfo.SCREEN_ORIENTATION_SENSOR);
         showInfo(R.string.unlocked, 1000);
         mLock.setBackgroundResource(R.drawable.ic_lock);
+        mTime.setEnabled(true);
+        mSeekbar.setEnabled(true);
+        mLength.setEnabled(true);
         mShowing = false;
         showOverlay();
     }
@@ -805,9 +813,9 @@ public class VideoPlayerActivity extends Activity {
             break;
 
         case MotionEvent.ACTION_MOVE:
-            // No audio/brightness action if coef < 2
+            // No volume/brightness action if coef < 2
             if (coef > 2) {
-                // Audio (Up or Down - Right side)
+                // Volume (Up or Down - Right side)
                 if (!mEnableBrightnessGesture || mTouchX > (screen.widthPixels / 2)){
                     doVolumeTouch(y_changed);
                 }
@@ -815,6 +823,12 @@ public class VideoPlayerActivity extends Activity {
                 if (mEnableBrightnessGesture && mTouchX < (screen.widthPixels / 2)){
                     doBrightnessTouch(y_changed);
                 }
+                // Extend the overlay for a little while, so that it doesn't
+                // disappear on the user if more adjustment is needed. This
+                // is because on devices with soft navigation (e.g. Galaxy
+                // Nexus), gestures can't be made without activating the UI.
+                if(Util.hasNavBar())
+                    showOverlay();
             }
             // Seek (Right or Left move)
             doSeekTouch(coef, xgesturesize, false);
@@ -874,7 +888,7 @@ public class VideoPlayerActivity extends Activity {
             mAudioManager.setStreamVolume(AudioManager.STREAM_MUSIC,
                     vol, 0);
             mIsAudioOrBrightnessChanged = true;
-            showInfo(getString(R.string.volume) + Integer.toString(vol),1000);
+            showInfo(getString(R.string.volume) + '\u00A0' + Integer.toString(vol),1000);
         }
     }
 
@@ -1192,9 +1206,10 @@ public class VideoPlayerActivity extends Activity {
             if (!mIsLocked) {
                 mOverlayHeader.setVisibility(View.VISIBLE);
                 mOverlayOption.setVisibility(View.VISIBLE);
-                mOverlay.setVisibility(View.VISIBLE);
+                mOverlayInterface.setVisibility(View.VISIBLE);
                 dimStatusBar(false);
             }
+            mOverlayProgress.setVisibility(View.VISIBLE);
         }
         Message msg = mHandler.obtainMessage(FADE_OUT);
         if (timeout != 0) {
@@ -1216,12 +1231,14 @@ public class VideoPlayerActivity extends Activity {
                 mOverlayLock.startAnimation(AnimationUtils.loadAnimation(this, android.R.anim.fade_out));
                 mOverlayHeader.startAnimation(AnimationUtils.loadAnimation(this, android.R.anim.fade_out));
                 mOverlayOption.startAnimation(AnimationUtils.loadAnimation(this, android.R.anim.fade_out));
-                mOverlay.startAnimation(AnimationUtils.loadAnimation(this, android.R.anim.fade_out));
+                mOverlayProgress.startAnimation(AnimationUtils.loadAnimation(this, android.R.anim.fade_out));
+                mOverlayInterface.startAnimation(AnimationUtils.loadAnimation(this, android.R.anim.fade_out));
             }
             mOverlayLock.setVisibility(View.INVISIBLE);
             mOverlayHeader.setVisibility(View.INVISIBLE);
             mOverlayOption.setVisibility(View.INVISIBLE);
-            mOverlay.setVisibility(View.INVISIBLE);
+            mOverlayProgress.setVisibility(View.INVISIBLE);
+            mOverlayInterface.setVisibility(View.INVISIBLE);
             mShowing = false;
             dimStatusBar(true);
         }
@@ -1340,13 +1357,15 @@ public class VideoPlayerActivity extends Activity {
         if (getIntent().getAction() != null
                 && getIntent().getAction().equals(Intent.ACTION_VIEW)) {
             /* Started from external application */
-            if(getIntent().getData().getScheme().equals("content")) {
+            if (getIntent().getData() != null
+                    && getIntent().getData().getScheme() != null
+                    && getIntent().getData().getScheme().equals("content")) {
                 if(getIntent().getData().getHost().equals("media")) {
                     // Media URI
                     Cursor cursor = managedQuery(getIntent().getData(), new String[]{ MediaStore.Video.Media.DATA }, null, null, null);
                     int column_index = cursor.getColumnIndexOrThrow(MediaStore.Video.Media.DATA);
-                    cursor.moveToFirst();
-                    mLocation = Util.PathToURI(cursor.getString(column_index));
+                    if (cursor.moveToFirst())
+                        mLocation = Util.PathToURI(cursor.getString(column_index));
                 } else {
                     // other content-based URI (probably file pickers)
                     mLocation = getIntent().getData().getPath();
